/**
 *  @file  <cal.h> 
 *  @brief functions to access to a calendaring 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */
#ifndef __LIPS_CALENDAR_H
#define __LIPS_CALENDAR_H

#include <stdint.h>

#include "caltype.h"
#include "calerror.h"
#include "calcomponent.h"



#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */


/**
 * @brief  type of  calendar.
 */
  typedef enum cal_type_t
  {
    CAL_TYPE_EVENT = 0, /**<      calendar        */
    CAL_TYPE_TODO,      /**<      todo/task       */
    CAL_TYPE_ALARM,     /**<      alarm           */
    CAL_TYPE_JOURNAL,    /**<      journal         */
    CAL_TYPE_TIMEZONE,    /**<      timezone         */
    CAL_TYPE_FREEBUSY    /**<      freebusy         */
  } cal_type_t;

/**
 * @brief  location of  calendar.
 */
typedef enum {
    CAL_LOCATION_LOCAL = 0,   /**< local  calendar*/
    CAL_LOCATION_NETWORK    /**<  remote calendar which locates on network*/
} cal_location_t;


/** 
 * @brief Used for modifying and removing instances. 
 */
  typedef enum
  {
    CAL_MOD_THIS = 1 << 0,
    CAL_MOD_THISANDPRIOR = 1 << 1,
    CAL_MOD_THISANDFUTURE = 1 << 2,
    CAL_MOD_ALL = 0x07
  } cal_mod_type_t;


/** 
 * @brief Used in cal_open_session() to connect to default calendar of some certain type 
 */
#define CAL_DEFAULT_EVENT ((uint32_t) -1)
#define CAL_DEFAULT_TODO ((uint32_t) -2)
#define CAL_DEFAULT_ALARM ((uint32_t) -3)
#define CAL_DEFAULT_JOURNAL ((uint32_t) -4)
#define CAL_DEFAULT_TIMEZONE ((uint32_t) -5)
#define CAL_DEFAULT_FREEBUSY ((uint32_t) -6)

/** @brief     Connect to a calendar and return the session id if connected successfully.
*  
* In this function, the calendar specified by "index" is found, 
* and be set ready to access. Applications could repeatedly call this function
* to connect to all the calendars available in the device.
*
* The macro "CAL_DEFAULT_EVENT", "CAL_DEFAULT_TODO", "CAL_DEFAULT_ALARM",
*  "CAL_DEFAULT_JOURNAL" "CAL_DEFAULT_TIMEZONE" "CAL_DEFAULT_FREEBUSY" can be 
* passed to "index" here to open the default calendar of the specific type.
*
* @param[in]  index       the index of the calendar. All calendars in the device 
*                           are continuously indexed from 0 to n.
* @param[out] sid         a pointer to the memory location where a session id is 
*                           returned, uniquely identifying the target calendar
*
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_INVALID         there's no calendar with the specifed index;
*           CAL_ERR_OUT_OF_RANGE    there's no calendar with an index equal to 
*                                    or higher than the specified one.
*/
  cal_err_t cal_open_session (uint32_t index, cal_sid_t *sid);



/** @brief     Disconnect the calendar used in the session specifed by the sid.
*
* @param[in]  sid       session id.
*
* @return   CAL_ERR_NONE         if  successful
*           CAL_ERR_*            other errors.
*/
  cal_err_t cal_close_session (cal_sid_t sid);

/** @brief     Return calendar name in this session.
* 
* Each calendar has some properties specified by the device management: name,
* product id, version, calendar scale.
*
* @param[in]     sid     session id.
* @param[out]    name    pointer to the memory location where a pointer to the 
*                        name string will be returned.
* 
* @return   CAL_ERR_NONE          if  successful
*           CAL_ERR_*             other errors.
*
* @note  The property has been loaded when "cal_open_session ()" is called,
*         and will be freed in the function "cal_close_session ()". Application 
*        doesn't need to free the memory.The same with following 
*        "cal_get_*()" function.
*/
  cal_err_t cal_get_name (cal_sid_t sid, const char **name);



/** @brief   Return the calendar product id.  
*  
* @param[in]    sid       session id.
* @param[out]   prodid    pointer to the memory location where the product id 
*                          will be returned.
*
* @return   CAL_ERR_NONE          if  successful
*           CAL_ERR_*             other errors.
*
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_prodid (cal_sid_t sid, const char **prodid);

/** @brief   Return the calendar version.
*
* @param[in]     sid          session id.
* @param[out]    version      pointer to the memory location where the version 
*                             will be returned.
*
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_*               other errors.
*
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_version (cal_sid_t sid, const char **version);

/** @brief   Return the calendar calendar scale.
*
* @param[in]    sid          session id.
* @param[out]   calscale     pointer to the memory location where the calscale will be returned.
*
* @return   CAL_ERR_NONE          if  successful
*           CAL_ERR_*             other errors.
*
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_calscale (cal_sid_t sid, const char **calscale);

/** @brief    Return the calendar type in this session.
*
* @param[in]    sid      session id.
* @param[out]   type     pointer to the memory location where an cal_type_t 
*                         value will be returned.
* 
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_*               other errors.
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_type (cal_sid_t sid, cal_type_t *type);


/** @brief    Return the location of the calendar in the session.
*
* @param[in]    sid      session id.
* @param[out]   location     pointer to the memory location where an cal_location_t 
*                         value will be returned.
* 
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_*               other errors.
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_location (cal_sid_t sid, cal_location_t *location);

/** @brief    Return the timezone id of calendar in the session.
*
* @param[in]     sid      session id.
* @param[out]   tzid     pointer to the memory location where an timezone id 
*                         			will be returned.
* 
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_*               other errors.
* @note     Application doesn't need to free the memory returned.
*/
  cal_err_t cal_get_timezone (cal_sid_t sid, const char **tzid);

/** @brief    Queries calendar for component corresponding to the given id. 
*
* @param[in]    sid         session id.
* @param[in]    uid         unique identifier for a component.
* @param[in]    rid         unique identifier for an instance in a recurrence 
*                            component;for non-recurrence component, it should be 
*                            set to NULL; for recurrence component, if it's set to
*                            NULL, the component will be returned as a whole;if 
*                            the "rid" is set to some value, the specified instance
*                            will be returned.
* @param[out]   comp        pointer to the memory location where the component 
*                           will be returned.
* @return   CAL_ERR_NONE              if  successful
*           CAL_ERR_NOT_FOUND          there's no component with the id.
*
* @note    Apps should later free the returned components by "cal_component_free ()".
*/
  cal_err_t cal_get_component_by_id (cal_sid_t sid,
                                      const char *uid,
                                      const char *rid,
                                      cal_component_t **comp);


/** @brief    Queries calendar for components which contain the word specified by "string". 
*
* @param[in]    sid         session id.
* @param[in]    string     the word to query.
* @param[out]  comps     pointer to the memory location where an array of components terminated with NULL will be returned.
*
* @return   CAL_ERR_NONE              if  successful
*           CAL_ERR_NOT_FOUND          there's no component with the id.
*
* @note    Apps should later free the returned components by "cal_component_free ()".
*/
   cal_err_t cal_get_components_by_fulltext (cal_sid_t sid,
   											 										 const char *string,
   											 										 cal_component_t ***comps);


   
/** @brief    Gets a list of components from the calendar that happen during the 
*             time specifed by "start" and "end".
* The "detach" parameter specify whether list only components or all instances.
* If detach == TRUE, get all the instances happenning in the period. The 
* instances in recurrence component will be detached.
* If detach == FALSE, as long as one ore more instances happen in the period, 
* the component wil be returned as a whole component, while the instances will
* not be detached.
*
* If start =0 and end !=0, this function will get components before "end".
* If start!=0 and end =0, this function will get components after "start".
* If start =0 and end =0, this function will get all components.
* If start !=0 and end !=0, this function will get components happen between 
*  "start" and "end".  
*
* @param[in]    sid            session id
* @param[in]    start          the time after which the component happen
* @param[in]    end            the time before which the component happen
* @param[in]    detach         indicate whether to list only components or all 
*                               instances
* @param[out]   comps           pointer to the memory location where the 
*                              components array will be returned, it's a NULL-terminated array.
* @return   CAL_ERR_NONE           if  successful
*           CAL_ERR_*               other errors.
*
* @note    Apps should later free the returned components by "cal_components_free ()". 
*/
  cal_err_t cal_get_components_by_time (cal_sid_t sid,
                                        time_t start,
                                        time_t end,
                                        cal_boolean_t detach,
                                        cal_component_t ***comps);


/** @brief     Add a component to the calendar. 
*
*  All clients will be notified with the "CAL_EVT_COMPONENT_ADDED" event. 
* 
* @param[in]   sid              session id.
* @param[in]   comp               pointer to the component to be added
* @param[out]  uid               if uid is not set to NULL, it will return the newly-allocated uid of the component. 
*
* @return   CAL_ERR_NONE              if  successful
*           CAL_ERR_INVALID           if "component"'s type is differrent from the calendar;
* @note    Apps should later free the returned uid
*/
  cal_err_t cal_add_component (cal_sid_t sid,
                               cal_component_t *comp,
                               char **uid);

/** @brief     Modify an existing component in the calendar. 
*  
*  For recurrence component, if an instance has been modified differrently from
*  other instances in the component, then this function will not apply to the 
*  differrent instance. See also cal_update_instance (sid, instance, CAL_MOD_ALL);
*  
*  All clients will be notified with the "CAL_EVT_COMPONENT_MODIFIED" event.
*
* @param[in]   sid              session id.
* @param[in]   comp               pointer to the component to be updated.
*
* @return   CAL_ERR_NONE              if  successful
*           CAL_ERR_NOT_FOUND          if the old component can't be found in the calendar.
*           CAL_ERR_INVALID           if "comp"'s type differrent from the calendar;
*/
  cal_err_t cal_update_component (cal_sid_t sid,
                                  const cal_component_t *comp);


/** @brief     Remove component from the calendar specified by "uid". 
*
*  All clients will be notified with the "CAL_EVT_COMPONENT_REMOVED" event.
*
* @param[in]   sid               session id.
* @param[in]   uid               the component uid 
* @return   CAL_ERR_NONE              if  successful
*           CAL_ERR_NOT_FOUND          there's no component with the uid;
*           CAL_ERR_*                 other errors.
*/
  cal_err_t cal_remove_component (cal_sid_t sid, 
                                  const char *uid);
                              
                              
/********************************************************************
 * instances related funcs                                          *
 ********************************************************************/                              

      
/** @brief     Modify an existing instance in the calendar. 
*
*  If the instance belongs to a recurrence component, the "modtype" will be 
*  checked; otherwise, the "modtype" will be ignored.
*  If the instance belongs to a recurrence component, and  modtype = CAL_MOD_ALL, 
*  this function will apply to all the instances in the original recurrence 
*  component, including the instances which have been modified differrently from
*  other instances in the component. See also cal_update_component();
*  If "instance" doesn't exist in the calendar, an error CAL_ERR_NOT_FOUND 
*  will be returned.
*  All clients will be notified with the "CAL_EVT_COMPONENT_MODIFIED" signal.
*
* @param[in]   sid             session id.
* @param[in]   instance           pointer to the instance to be updated.
* @param[in]   modtype         if modtype=CAL_MOD_THIS, only update the instance;
*                          if modtype=CAL_MOD_ALL,all instances in the component
*                          will be update;
*                          if modtype=CAL_MOD_THISANDPRIOR,the instances on and
*                          before it will be update;
*                          if modtype=CAL_MOD_THISANDFUTURE,the instances on and
*                          after it will be update;
*
* @return   CAL_ERR_NONE            if  successful
*           CAL_ERR_NOT_FOUND        if the instance can't be found in the calendar
*           CAL_ERR_INVALID         if instance's type differrent from the calendar
*/
cal_err_t cal_update_instance (cal_sid_t sid, 
                              const cal_component_t *instance,
                              cal_mod_type_t modtype);
                              
/** @brief     Remove instance from the calendar specified by "uid". 
*
*  If the instance belongs to a recurrence component, the "modtype" will be 
*  checked; otherwise, the "modtype" will be ignored.
*  If the instance doesn't exist in the calendar, an error CAL_ERR_NOT_FOUND 
*  will be returned.
*  All clients will be notified with the "CAL_EVT_COMPONENT_REMOVED" signal.
*
* @param[in]   sid                 session id.
* @param[in]   uid                 uid of the instance.
* @param[in]   rid                 rid of the instance 
* @param[in]   modtype             if modtype=CAL_MOD_THIS, only remove the instance;
*                                   if modtype=CAL_MOD_ALL,all instances in the component
*                                   will be removed;
*                                   if modtype=CAL_MOD_THISANDPRIOR,the instances on and
*                                   before it will be removed;
*                                   if modtype=CAL_MOD_THISANDFUTURE,the instances on and
*                                   after it will be removed;
* @return   CAL_ERR_NONE           if  successful
*           CAL_ERR_NOT_FOUND       there's no instance with the uid;
*           CAL_ERR_*              other errors.
*/                                                                      
cal_err_t cal_remove_instance (cal_sid_t sid, 
                                const char *uid,
                                const char *rid,
                                cal_mod_type_t modtype);



/** @brief     decides whether the instance belongs to a recurrence component or a 
* non-recurrence component.
*
*
* @param[in]    sid          session id.
* @param[in]    instance     pointer to an instance.
* @param[out]   bool         return value; if the instance belongs to a recurrence component
*														 ,the return value is TRUE; otherwise, it's FALSE.
*
* @return   CAL_ERR_NONE            if  successful;
*           CAL_ERR_*               other errors.
*/
  cal_err_t cal_instance_is_multiple (cal_sid_t sid, 
                                      const cal_component_t *instance,
                                      cal_boolean_t *bool);
                                                                      

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_CALENDAR_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
