/**  
 *  @file  <calevent.h> 
 *  @brief provide event mechanism
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *    notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */
#ifndef  __LIPS_CAL_EVENT_H
#define  __LIPS_CAL_EVENT_H


#include "caltype.h"
#include "calerror.h"

#ifdef __cplusplus
extern "C" { 
#endif /* __cplusplus */

/** 
* @brief calendaring event type
*/
  typedef enum {
    
    CAL_EVT_COMPONENT_ADDED = 1,    /**<unsolicited event: new component added */
    CAL_EVT_COMPONENT_MODIFIED,      /**<unsolicited event: component modified */
    CAL_EVT_COMPONENT_REMOVED,      /**<unsolicited event: component removed */
    CAL_EVT_COMPONENT_RECEIVED,      /**<unsolicited event: component received */
    CAL_ALARM_TIME_REACHED,		/**<unsolicited event: alarm time reached */
    
    CAL_REQ_SEND_COMPONENT          /**<asynchronous event: send out a group scheduling component */
  } cal_evt_type_t;
 
/**
* @brief indicate whether send out successfully or not
*/
  typedef enum {
    CAL_ERR_CODE_NONE,              /**< send out successful*/
    CAL_ERR_CODE_FAILED             /**< fail to send out the component*/
  } cal_err_code_t;

	
/**
 * @struct cal_evt_async_t
 * @brief Event asynchronous data structure
 */
  typedef struct
  {
    cal_err_code_t err_code;         /**< Error Code */
    uint32_t rid;              /**< Request Identifier */
	 char *attendee;                         /**< The attendee address to which send the component*/
  } cal_evt_async_t;


  typedef struct _cal_evt_t cal_evt_t;

  typedef void (*cal_cb_t) (cal_sid_t sid, cal_evt_t *evt, void *user_data);
  
/**
 * @struct ca_evt_t
 * @brief  a structure to hold an event.
 */
  struct _cal_evt_t
  {
    cal_evt_type_t evt_type;                /**< Event ID */
    cal_sid_t sid;                          /**< Session ID */
    cal_cb_t *cb;                           /**< Call back */
    char *uid;                              /**< The component uid */    
    cal_evt_async_t *async_data;            /**< Asynchronous related information */
  }; 

  
  
/******************evt handling functions********************/
/**
 *@brief register an unsolicited event.
 *
 *Before application can receive any unsolicited event, it should
 *register the event type by this function.
 *
 *@param[in]  sid       target session id.
 *@param[in]  type       the unsolicited event type, refer to cal_evt_type_t.
 *@param[in]  cb         the callback for the incoming events.
 *@return     CAL_ERR_NONE         if  successful
 *             CAL_ERR_*            other errors.
 */

  cal_err_t cal_evt_register (cal_sid_t sid,
                              cal_evt_type_t type, 
                              cal_cb_t *cb);
/**
 *@brief unregister an unsolicited event.
 *
 *If application don't want to receive a kind of unsolicited event any
 *more, it should use this function to un-register the event type.
 *
 *@param[in]  sid       target session id.
 *@param[in]  type       the unsolicited event type.
 *@return     CAL_ERR_NONE         if  successful
 *             CAL_ERR_*            other errors. 
 */

  cal_err_t cal_evt_unregister (cal_sid_t sid, cal_evt_type_t type);
  
  

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __LIPS_CAL_EVENT_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
