/**
 *  @file  <calgschedule.h> 
 *  @brief functions to operate on group scheduling component
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */
#ifndef _LIPS_CAL_COMPONENT_H
#define _LIPS_CAL_COMPONENT_H

#include  "caltype.h"
#include  "calerror.h"
#include  "calcomponent.h"


#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */


#define DEFAULT_EMAIL_CHANNEL "email_channel"

/**************************************
* Scheduling functions                *
***************************************/

/** @brief     List unhandled component. 
 *
 * The newly-coming component will be stored into an unhandled component table temporarily.
 * When "cal_accept_component()" or "cal_deny_component()" is called, the 
 * unhandled component table will be updated.  
 *
 * @param[in]   sid           session id.
 * @param[in]   comps         pointer to an array of unread component 
 *
 * @return   CAL_ERR_NONE           if successful
 */
cal_err_t cal_list_unhandled_components (cal_sid_t sid, cal_component_t ***comps);

/** @brief     Accept a component. 
 *
 * The local data storage and the unread component table will be updated accordingly. 
 *
 * @param[in]   sid           session id.
 * @param[in]   comp          pointer to the component 
 *
 * @return   CAL_ERR_NONE           if successful
 */
cal_err_t cal_accept_component (cal_sid_t sid, cal_component_t *comp);

  
/** @brief     Deny a component. 
 *
 * The local data storage and the unread component table will be updated accordingly. 
 * 
 * @param[in]   sid           session id.
 * @param[in]   comp          pointer to the component 
 *
 * @return   CAL_ERR_NONE           if successful
 */
cal_err_t cal_deny_component(cal_sid_t sid, cal_component_t *comp);



/** @brief     Send out a component.
*
*The function generates an CAL_REQ_SEND_COMPONENT event. The event will be
*sent back after a couple of time, and the callback function will be called.
* 
* @param[in]   sid           session id.
* @param[in]   cb            callback function for asynchronous call  
* @param[in]   comp        pointer to the component to be send out
* @param[in]   channel	   the channel to send the component
* @param[out] rid 			   request identifier
*
* @return   CAL_ERR_NONE           if send out successfully;
*            CAL_ERR_INVALID          if the component doesn't conform to RFC2446.
*/
cal_err_t cal_send_component (cal_sid_t sid, cal_cb_t *cb,
                              const cal_component_t *comp,  cal_channel_t channel, uint32_t *rid);

  
/** @brief     Cancel the sending of a component.
*
* 
* @param[in]   sid           session id.
* @param[out] rid 			   request identifier
*
* @return   CAL_ERR_NONE           if successful
*/
cal_err_t cal_send_component_cancel(cal_sid_t sid,  uint32_t rid);


#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* _LIPS_CAL_COMPONENT_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/
