/*  libcalenabler2 - Library of LiPS calendar enabler
 *
 *Authors: Yang Lin <lin.yang@orange-ftgroup.com>
 *
 *This software and associated documentation files (the "Software") 
 *are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *All Rights Reserved. 
 *
 *A copyright license is hereby granted for redistribution and use of 
 *the Software in source and binary forms, with or without modification, 
 *provided that the following conditions are met: 
 *- Redistributions of source code must retain the above copyright notice, 
 *this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *notice, this copyright license and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution. 
 *- Neither the name of LiPS  nor the names of its Members may be used 
 *to endorse or promote products derived from the Software without 
 *specific prior written permission. 
 *
 *A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *distribute any implementation compliant with the any specification adopted 
 *by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *and non-discriminatory terms and conditions and under reciprocity, as 
 *regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *POSSIBILITY OF SUCH DAMAGE. 
 */
 

#include <time.h> /* for time() */
#include <glib.h>
#include <sqlite3.h>
#include <uuid/uuid.h>


#include <ical.h>

#include <calcomponent.h>
#include <caltype.h>
#include <calerror.h>

#include "cal-priv.h"
#include "calcomponent-priv.h"
#include "cal-db.h"
#include "calenv.h"
#include "calicalutils.h"

#define COMPONENT_TABLE_CREATION_SQL  "create table if not exists component" \
                                  " (uid TEXT," \
                                  " rid TEXT," \
                                  " start INTEGER," \
                                  " end INTEGER," \
                                  " rdate TEXT," \
                                  " rrule TEXT," \
                                  " exdate TEXT," \
                                  " due INTEGER," \
                                  " duration INTEGER," \
                                  " icalstring TEXT" \
                                  ");"
#define TIMEZONE_TABLE_CREATION_SQL 	"create table if not exists timezone" \
                                  " (tzid TEXT," \
                                  " icalstring TEXT" \
                                  ");"																
#define INSERT_COMPONENT_SQL      "insert into component " \
																	"(uid,rid,start,end,rdate,rrule,exdate,due,duration,icalstring)" \
																  " values ('%q','%q',%d,%d,'%q','%q','%q',%d,%d,'%q');"
																					
#define REMOVE_COMPONENT_SQL      "delete from component where uid='%q';"
#define REMOVE_MASTER_COMPONENT_SQL      "delete from component where uid='%q' AND (rid ISNULL);"
#define REMOVE_INSTANCE_SQL       "delete from component where uid='%q' AND rid='%q';"


typedef enum 
	{
	ALL_COMPONENT = 0,
  ONLY_MASTER_COMPONENT		
	} rm_flag_t;
	
/* define calendar struct to store the overall properties */
cal_err_t cal_initialize_db (CalBackend *bk)
{
	CalBackendDb *priv = NULL;
	gchar *db_path = NULL;
	gchar *err_str;
  gint ret;

  g_assert (bk);

	priv = g_new0 (CalBackendDb, 1);
  if (priv == NULL)
  {
    return CAL_ERR_MEM_ALLOC;
  }

  db_path = g_build_filename (_env_get_source_prefix_db(), bk->source, NULL);
	
	if (sqlite3_open (db_path, &priv->component_db))
  {
    g_print ("%s(): component database error.[%s]\n", __FUNCTION__, db_path);
    sqlite3_close (priv->component_db);
    g_free (db_path);
    g_free (priv);

    return CAL_ERR_DB_TRANS;
  }
	
	ret =
    sqlite3_exec (priv->component_db, COMPONENT_TABLE_CREATION_SQL, NULL, NULL,
                  &err_str);
	if (ret != SQLITE_OK)
  {
    g_print ("%s(): component table creation error. [%s]\n", __FUNCTION__, err_str);
    g_free (err_str);
    return CAL_ERR_DB_TRANS;
  }
	
	ret =
    sqlite3_exec (priv->component_db, TIMEZONE_TABLE_CREATION_SQL, NULL, NULL,
                  &err_str);
	if (ret != SQLITE_OK)
  {
    g_print ("%s(): timezone table creation error. [%s]\n", __FUNCTION__, err_str);
    g_free (err_str);
    return CAL_ERR_DB_TRANS;
  }
	bk->priv = (gpointer) priv;
	
	g_free (db_path);
  return CAL_ERR_NONE;
}

void
cal_finalize_db (CalBackend *bk)
{
  CalBackendDb *priv = NULL;

  g_assert (bk);

  g_print ("%s():\n", __FUNCTION__);

  priv = (CalBackendDb *) bk->priv;

  if (priv->component_db)
    sqlite3_close (priv->component_db);

  if (priv->subcomponent_db)
    sqlite3_close (priv->subcomponent_db);

  g_free (priv);

  g_print ("%s(): Leaving\n", __FUNCTION__);

  return;	
}


/*Add a component structure into database. The structure can be either component 
or instance. The function won't deal with "uid",which is processed in cal_add_component()*/
static cal_err_t _write_event_component_into_db (sqlite3 *db, cal_component_t *comp)
{	
	g_return_val_if_fail (comp!=NULL, CAL_ERR_INVALID);
	
	int i;
	/*used for single-value properties*/
	const gchar *uid = NULL;
	const gchar *rid = NULL;
	const gchar *dtstart = NULL;
	const gchar *dtend = NULL;
	const gchar *due = NULL;
	time_t start_t, end_t, due_t;
	int32_t duration = 0;
	const gchar *recur = NULL;
	//gchar *str_recur = NULL;
	
	/*used for multiple-value properties*/
	uint32_t *rdate_pids = NULL;  
	uint32_t *exdate_pids = NULL;
	GString *rdate_list = g_string_new (NULL);
	GString *exdate_list = g_string_new (NULL);	
	const gchar *rdate = NULL;
	const gchar *exdate = NULL;
	
	GString *insert_str = g_string_new ("insert into component values ('");
	sqlite3_stmt *pstmt = NULL;
	gint ret;	
	

		
/**** uid ****/	
	cal_proptype_get_string_value (comp, CAL_UID_PROPERTY, &uid);
	/*uid must be set*/
	if (!uid)
		return CAL_ERR_INVALID;
	
	g_string_append (insert_str, uid);
	g_string_append (insert_str, "', ");
	
/**** rid ****/	
	cal_proptype_get_string_value (comp, CAL_RECURRENCEID_PROPERTY, &rid);
	if (rid)
		{	
		g_string_append (insert_str, "'");	
		g_string_append (insert_str , rid);	
		g_string_append (insert_str, "', ");
		}
	else
		g_string_append (insert_str, "NULL,");
		
/**** start ****/				
	cal_proptype_get_string_value (comp, CAL_DTSTART_PROPERTY, &dtstart);
	/*dtstart must be set*/
	if (!dtstart)
		return CAL_ERR_INVALID;
	start_t = icaltime_as_timet (icaltime_from_string (dtstart));
	char *startstr = g_malloc0 (20);	
	sprintf (startstr, "%d", start_t);
	g_string_append (insert_str , startstr);
	g_string_append (insert_str, ",");
	g_free (startstr);
	
/**** end ****/		
	cal_proptype_get_string_value (comp, CAL_DTEND_PROPERTY, &dtend);
	if (dtend)
		{
			char *endstr = g_malloc0 (20);
			end_t = icaltime_as_timet (icaltime_from_string (dtend));
			sprintf (endstr, "%d", end_t);
			g_string_append (insert_str, endstr);
			g_string_append (insert_str, ",");
			g_free (endstr);
		}
	else
		g_string_append (insert_str, "NULL,");
	
/**** rdate ****/		
	cal_component_get_properties (comp, CAL_RDATE_PROPERTY, &rdate_pids);
	if (rdate_pids)
	{
		for (i=0; rdate_pids[i]!=PID_INVALID_VALUE; i++)
		{
			cal_property_get_string_value(comp, rdate_pids[i], &rdate);
			g_string_append (rdate_list, rdate);
			g_string_append (rdate_list, ",");
		}
		g_string_erase (rdate_list, rdate_list->len - 1,1);
		g_free (rdate_pids);
		g_string_append (insert_str, "'");
		g_string_append (insert_str, rdate_list->str);
		g_string_append (insert_str, "', ");
		
	}		
	else
		g_string_append (insert_str, "NULL,");
	
/**** rrule ****/	
/*If rid != NULL, this component is a detached component for one instance, there
	should be rrule == NULL, exdate == NULL*/
	if (rid != NULL)
	{
		g_string_append (insert_str, "NULL,");
	}
	else
	{
		
		cal_proptype_get_string_value (comp, CAL_RRULE_PROPERTY, &recur);
		if (recur != NULL)
		{
			
			g_string_append (insert_str, "'");
			g_string_append (insert_str, recur);
			g_string_append (insert_str, "', ");
		}
		else
			g_string_append (insert_str, "NULL,");
	}
	
/**** exdate ****/		
	if (rid != NULL)
	{
		g_string_append (insert_str, "NULL,");
	}
	else
	{
		cal_component_get_properties (comp, CAL_EXDATE_PROPERTY, &exdate_pids);
		if (exdate_pids)
		{
			for (i=0; exdate_pids[i]!=PID_INVALID_VALUE; i++)
			{
				cal_property_get_string_value (comp, exdate_pids[i], &exdate);
				g_string_append (exdate_list, exdate);
				g_string_append (exdate_list, ",");
	#ifdef DEBUG
		g_print ("%s():%d -- exdate = %s\n", __FUNCTION__, __LINE__, exdate);
	#endif			
			}		
			g_string_erase (exdate_list, exdate_list->len - 1,1);
			g_free (exdate_pids);
			g_string_append (insert_str, "'");
			g_string_append (insert_str, exdate_list->str);
			g_string_append (insert_str, "', ");
		}
		else
			g_string_append (insert_str, "NULL,");
	}
/**** due ****/
	cal_proptype_get_string_value (comp, CAL_DUE_PROPERTY, &due);	
	if (due)
		{
		due_t = icaltime_as_timet (icaltime_from_string (due));
			
		char *duestr = g_malloc0 (20);	
		sprintf (duestr, "%d", due_t);
		g_string_append (insert_str , duestr);
		g_string_append (insert_str, ",");
		g_free (duestr);
		}
	else
		g_string_append (insert_str, "NULL,");
 
/**** duration ****/	
	cal_proptype_get_int32_value (comp, CAL_DURATION_PROPERTY, &duration);
	if (duration != 0)
		{
		char *durationstr = g_malloc0 (20);	
		sprintf (durationstr, "%d", duration);
		g_string_append (insert_str , durationstr);
		g_string_append (insert_str, ",");
		g_free (durationstr);
		}
	else
		g_string_append (insert_str, "NULL,");
	

/*** check the validity before add into database ***/	
	if ((dtend) && (duration != 0))
		if ((end_t - start_t) != duration)	
		{
			g_print ("%s():%d: the start,end and duration contradict\n", __FUNCTION__, __LINE__);
			return CAL_ERR_INVALID;
		}
		
/*only master component which has rrule could have rdate and exdate*/
	if ((recur == NULL) &&
			((rdate != NULL) || (exdate != NULL)))
		{
			g_print ("%s():%d: the rrule,rdate and exdate contradict\n", __FUNCTION__, __LINE__);
			return CAL_ERR_INVALID;
		}
/*a detached component shouldn't have rrule and exdate*/		
	if ((rid != NULL) && (recur != NULL))
		{
			g_print ("%s():%d: the rid,rrule contradict\n", __FUNCTION__, __LINE__);
			return CAL_ERR_INVALID;	
		}
		
/**** icalstring****/	
	g_string_append (insert_str, "'");
	g_string_append (insert_str, icalcomponent_as_ical_string (comp->icalcomp));
	g_string_append (insert_str, "');");

	
#ifdef DEBUG
	g_print ("%s(): %d insert string: %s\n", __FUNCTION__, __LINE__, insert_str->str);
#endif

	sqlite3_prepare (db, insert_str->str, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);
#ifdef DEBUG
	g_print ("sqlite3 bind blob status :%d. \n", ret);
#endif
	sqlite3_finalize (pstmt);
	

 	//g_free (str_recur);
	g_string_free (rdate_list, TRUE);

	g_string_free (exdate_list, TRUE);
	g_string_free (insert_str, TRUE);
#ifdef DEBUG
	g_print ("%s():%d \n", __FUNCTION__, __LINE__);
#endif 	
if (ret == SQLITE_DONE)
		return CAL_ERR_NONE;
	else
		{
		g_print ("%s():%d, ret = %d\n", __FUNCTION__, __LINE__, ret);
		return CAL_ERR_DB_TRANS;
		}
}


static cal_err_t _write_timezone_component_into_db (sqlite3 *db, cal_component_t *comp)
{
	g_return_val_if_fail (comp!=NULL, CAL_ERR_INVALID);
	
	GString *insert_str = g_string_new ("insert into timezone values ('");
	sqlite3_stmt *pstmt = NULL;
	const gchar *tzid;
	gint ret;
	
	cal_proptype_get_string_value (comp, CAL_TZID_PROPERTY, &tzid);
	/*uid must be set*/
	if (!tzid)
		return CAL_ERR_INVALID;
	
	g_string_append (insert_str, tzid);
	g_string_append (insert_str, "', ");
	
	g_string_append (insert_str, "'");
	g_string_append (insert_str, icalcomponent_as_ical_string (comp->icalcomp));
	g_string_append (insert_str, "');");

	
#ifdef DEBUG
	g_print ("%s(): %d insert string: %s\n", __FUNCTION__, __LINE__, insert_str->str);
#endif

	sqlite3_prepare (db, insert_str->str, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);
#ifdef DEBUG
	g_print ("sqlite3 bind blob status :%d. \n", ret);
#endif
	sqlite3_finalize (pstmt);
	g_string_free (insert_str, TRUE);

if (ret == SQLITE_DONE)
		return CAL_ERR_NONE;
	else
		{
		g_print ("%s():%d, ret = %d\n", __FUNCTION__, __LINE__, ret);
		return CAL_ERR_DB_TRANS;
		}
	
}

static cal_err_t _write_component_into_db (sqlite3 *db, cal_component_t *comp)
{
	cal_comp_type_t ctype;

	cal_component_get_type (comp, &ctype);
	
	switch (ctype)
		{
			case CAL_EVENT_COMPONENT:
				return 	_write_event_component_into_db(db, comp);
			case CAL_TIMEZONE_COMPONENT:
				return _write_timezone_component_into_db(db, comp);
			default:
				return CAL_ERR_INVALID;
		}
}
cal_err_t cal_add_component_db (CalBackend *bk ,cal_component_t *component, char **uid)
{
	CalBackendDb * priv = NULL;
	uuid_t uuid;
  gchar newuid[36];
	const gchar *temp_uid = NULL;
	cal_comp_type_t ctype;
	cal_err_t ret;
	
	g_return_val_if_fail (component!=NULL, CAL_ERR_INVALID);
	
	cal_component_get_type (component, &ctype);
	
	switch (bk->type)
	{	
		case CAL_TYPE_EVENT:
			if ((ctype != CAL_EVENT_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
				return CAL_ERR_INVALID;
			break;
		case CAL_TYPE_TODO:
			if ((ctype != CAL_TODO_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
					return CAL_ERR_INVALID;
			break;
		case CAL_TYPE_ALARM:
			if ((ctype != CAL_EVENT_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
					return CAL_ERR_INVALID;
			break;
		case CAL_TYPE_JOURNAL:
			if ((ctype != CAL_JOURNAL_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
					return CAL_ERR_INVALID;	
			break;
	}
	
	priv = (CalBackendDb *)bk->priv;
	
	if (ctype == CAL_EVENT_COMPONENT)
	{
		cal_proptype_get_string_value (component, CAL_UID_PROPERTY, &temp_uid);
		if (temp_uid != NULL)
		{
			g_print ("The event component uid should be allocated by enabler in the function cal_add_component()\n");
			return CAL_ERR_INVALID;
		}	
		 
		/*generate uid*/
		uuid_generate (uuid);
		uuid_unparse (uuid, newuid);
		
		cal_proptype_set_string_value (component, CAL_UID_PROPERTY, newuid);
		if (uid != NULL)
			*uid = g_strdup (newuid);
	
		return _write_event_component_into_db (priv->component_db, component);
	}
	else if (ctype == CAL_TIMEZONE_COMPONENT)
		{
			cal_proptype_get_string_value (component, CAL_TZID_PROPERTY, &temp_uid);
			if (temp_uid == NULL)
				{
				g_print ("The time zone component tzid should be set before add into calendar\n");
				return CAL_ERR_INVALID;					
				}
			
			if (uid != NULL)
				*uid = g_strdup (temp_uid);
			return _write_timezone_component_into_db (priv->component_db, component);
		}
	else if (ctype == CAL_TODO_COMPONENT)
	{
		/*FIXME: add support*/
	}
	else if (ctype == CAL_JOURNAL_COMPONENT)
	{
		/*FIXME: add support*/
	}
	
}


/*Get a result from database query, transform the result to component structure 
and insert it into an array*/
static void	
_fill_components (sqlite3_stmt *pstmt, GPtrArray *array)
{
	gchar *value  = NULL;
	cal_component_t *tempcomp ;
			
	value = (gchar*)sqlite3_column_text (pstmt, 0);
#ifdef DEBUG
	g_print ("%s (): icalstring = %s\n", __FUNCTION__, value);
#endif	
	
	_cal_component_new_from_icalcomponent (&tempcomp, icalcomponent_new_from_string (value));
	g_ptr_array_add (array, tempcomp);

}

/*called by the function cal_get_components_by_time_db() */
static cal_err_t
_select_non_repeating_components (sqlite3 *db, time_t start, time_t end, GPtrArray *array)
{
	GString *sql_str = NULL;
	gchar *condition_str = NULL;
	sqlite3_stmt *pstmt = 0;
	gint ret;
	
	
	//sql_str = g_string_new ("select icalstring from component where rrule ISNULL" \
	//												" AND rdate ISNULL AND rid ISNULL ");
	
	sql_str = g_string_new ("select icalstring from component where rrule ISNULL" \
													" AND rid ISNULL ");
													
	if (start != 0)
  {
    if (end != 0)
    {
      condition_str = sqlite3_mprintf ("AND (start <= %d AND end >= %d);", end, start);
    }
    else
    {
      condition_str = sqlite3_mprintf ("AND (end >= %d);",start);
    }
  }
	else
  {
    if (end != 0)
    {
      condition_str = sqlite3_mprintf ("AND (start <= %d);", end);
    }
    else
    {
      condition_str = sqlite3_mprintf (";");
    }
  }

#if 0
   	if (start != 0)
  {
    if (end != 0)
   	 condition_str = sqlite3_mprintf ("AND (start >= %d AND start <= %d);", start, end);
    else
     condition_str = sqlite3_mprintf ("AND (start >= %d);",start);
  }
	else
  {
    if (end != 0)
    	condition_str = sqlite3_mprintf ("AND (start <= %d);", end);
    else
      condition_str = sqlite3_mprintf (";");
  }
#endif
  g_string_append (sql_str, condition_str);

#ifdef DEBUG
	//g_print ("%s(): sql_str = %s\n", __FUNCTION__, sql_str->str);
#endif	
	
	ret = sqlite3_prepare (db, sql_str->str, -1, &pstmt, 0);
		
	while ((ret = sqlite3_step (pstmt)) == SQLITE_ROW)
		_fill_components (pstmt, array);
	
	ret = sqlite3_finalize (pstmt);
  if (ret != SQLITE_OK)
  {
    return CAL_ERR_DB_TRANS;
  }
	g_free (condition_str);
	g_string_free (sql_str, TRUE);
		
	return CAL_ERR_NONE;
}

cal_err_t _fill_generated_occurrence (cal_component_t *comp, const char * rid, cal_component_t **instance)
{
	gchar *value = NULL;
	int i;
	int interval = 0;
	const gchar *start = NULL;
	const gchar *end = NULL;
	time_t start_t, end_t; 
	cal_component_t *tempoccur = NULL;
	time_t rid_time;
	
#ifdef DEBUG
	g_print ("%s():%d\n", __FUNCTION__, __LINE__);
#endif
	
	cal_proptype_get_string_value (comp, CAL_DTEND_PROPERTY, &end);
	
	if (end)
	{
		cal_proptype_get_string_value (comp, CAL_DTSTART_PROPERTY, &start);
		/*FIXME: Can icaltime_from_string parse parameter such as "tzid"*/
		end_t = icaltime_as_timet (icaltime_from_string (end));
		start_t = icaltime_as_timet (icaltime_from_string (start));
		interval = end_t - start_t;
	}
	if (1)
	{		
				
		_cal_component_new_clone (comp, &tempoccur);
		/*FIXME: Expand return time_t indicating start time, so get rid from it*/
	
		cal_proptype_set_string_value (tempoccur, CAL_RECURRENCEID_PROPERTY, rid);
		/*detached component's rrule is NULL*/
		//cal_component_set_string_property_value (tempoccur, RRULE_FIRST_PID, NULL);
		
		/*detached component which represent one instance shouldn't have rrule*/
		uint32_t *pids1 = NULL, *pids2 = NULL;
		int i=0;
		cal_component_get_properties (tempoccur, CAL_RRULE_PROPERTY, &pids1);
		
		if (pids1 != NULL)
			{
			cal_component_remove_property_by_type (tempoccur, CAL_RRULE_PROPERTY);
			g_free (pids1);
			}	
					
		cal_component_get_properties (tempoccur, CAL_EXDATE_PROPERTY, &pids2);
		if (pids2 != NULL)
			{
				i = 0;
				cal_component_remove_property_by_type (tempoccur, CAL_EXDATE_PROPERTY);
				g_free (pids2);
			}
				
		start_t = icaltime_as_timet ( icaltime_from_string (rid));
		start = icaltime_as_ical_string (icaltime_from_timet (start_t, 0));
	
		cal_proptype_set_string_value (tempoccur, CAL_DTSTART_PROPERTY, start);
			
		if (end)
		{
			end_t = start_t + interval;
			end = icaltime_as_ical_string (icaltime_from_timet (end_t, 0));
			cal_proptype_set_string_value (tempoccur, CAL_DTEND_PROPERTY, end);
		}
#ifdef DEBUG
	g_print ("%s():%d, rid=%s, start=%s, end = %s\n", __FUNCTION__, __LINE__, rid, start, end);
#endif		

		*instance = tempoccur;
	}
		

	return CAL_ERR_NONE;	
}

/*according to generated rids , construct instances from master component*/
static cal_err_t
_fill_generated_occurrences (cal_component_t *comp, const GPtrArray *occur_rid, GPtrArray *array)
{
	gchar *value = NULL;
	int i;
	int interval = 0;
	const gchar *start = NULL;
	const gchar *end = NULL;
	time_t start_t, end_t; 
	const gchar *rid = NULL;
	cal_component_t *tempoccur = NULL;

#ifdef DEBUG
	g_print ("%s():%d\n", __FUNCTION__, __LINE__);
#endif
	
	
	for (i=0; i<occur_rid->len; i++)
	{		
		time_t rid;
		char *ridstr;

		rid = *((time_t *)g_ptr_array_index (occur_rid, i));
		/** set rid, start, end for each instance **/
		ridstr = icaltime_as_ical_string (icaltime_from_timet (rid, 0));
#ifdef DEBUG
	g_print ("%s():%d, rid=%d\n", __FUNCTION__, __LINE__, rid);
#endif	
		_fill_generated_occurrence (comp, ridstr, &tempoccur);

		g_ptr_array_add (array, tempoccur);	
	

	}
		

	return CAL_ERR_NONE;
}
/*parse the exdatestring to get the seperated exdate, the seperated exdate's type should be time_t*/
GPtrArray *_parse_exdate_string(gchar *exdate)
{
	GPtrArray *gparray = g_ptr_array_new();
	gchar *temp = g_new(gchar, 16);
	gchar *rtemp = temp;
	time_t *t = (time_t *)g_malloc0 (sizeof(time_t));
	
	while (*exdate)
	{
		if (*exdate ==',')
		{
			*temp = '\0';
			*t = icaltime_as_timet ( icaltime_from_string (rtemp));
			g_ptr_array_add(gparray, (gpointer)t);
			t = (time_t *)g_malloc0 (sizeof(time_t));
			exdate++;
			temp = rtemp;			
		}
		*temp++ = *exdate++;
	}
	*temp = '\0';
	*t = icaltime_as_timet ( icaltime_from_string (rtemp));
	g_ptr_array_add(gparray, (gpointer)t);	
	g_free(rtemp);
#ifdef DEBUG
	int k;
	for (k=0; k<gparray->len; k++)
		g_print ("%d():%d gparray = %d\n", __FUNCTION__, __LINE__, *(time_t *)g_ptr_array_index(gparray, k));
#endif	
	return gparray;
	
}

/*called by the function cal_get_components_by_time_db() */
static cal_err_t
_select_repeating_components (sqlite3 *db, time_t start, time_t end, gboolean detached, GPtrArray *array)
{
	gchar *sql_str_uid = NULL;
	gchar *condition_str = NULL;	
	sqlite3_stmt *pstmt_uid = 0;	
	const gchar *str_uid = NULL;	
	cal_component_t *comp = NULL;
	gint ret;

	
/** Prepare condition_str which will be used in later process **/		

	if (start != 0)
  {
    if (end != 0)
   	 condition_str = sqlite3_mprintf ("AND (start <= %d AND end >= %d);", end, start);
    else
     condition_str = sqlite3_mprintf ("AND (end >= %d);",start);
  }
	else
  {
    if (end != 0)
    	condition_str = sqlite3_mprintf ("AND (start <= %d);", end);
    else
      condition_str = sqlite3_mprintf (";");
  }


	
/** Find the master component for repeating component **/	
	sql_str_uid = sqlite3_mprintf ("select distinct uid from component where " \
	                                      "rrule NOTNULL;");
	sqlite3_prepare (db, sql_str_uid, -1, &pstmt_uid, 0);	
	/** Deal with each uid **/
	while (ret = sqlite3_step (pstmt_uid) == SQLITE_ROW)
	{
		/** rid_generated used to store the rid of the instances which can be 
		generated from the master component **/
		GPtrArray *rid_generated = NULL;  
		
		sqlite3_stmt *pstmt_icalstring = 0;
		sqlite3_stmt *pstmt_rid = 0;
		gchar *sql_str_icalstring = NULL;		
		GString *sql_str_rid = NULL;
		const gchar *str_icalstring = NULL;
		const gchar *str_rid = NULL;
	  const gchar *str_exdate = NULL;
		GPtrArray *array_exdate = NULL;
		
		str_uid = (const gchar *)sqlite3_column_text (pstmt_uid, 0);	
		
		/** get the master component**/
		sql_str_icalstring = sqlite3_mprintf ("select icalstring, exdate from component where " \
		                                      "uid='%q' AND rid ISNULL;", str_uid);
#ifdef DEBUG
		g_print ("%s():%d sql_str_icalstring = %s\n", __FUNCTION__, __LINE__, sql_str_icalstring);
#endif		
		
		sqlite3_prepare (db, sql_str_icalstring, -1, &pstmt_icalstring, 0); 
		sqlite3_step (pstmt_icalstring);
		
		str_icalstring = (const gchar *)sqlite3_column_text (pstmt_icalstring, 0);
		/**construct cal_component_t structure from the icalstring**/
		_cal_component_new_from_icalcomponent (&comp, icalcomponent_new_from_string (str_icalstring));
		
		if ((start == 0) && (end == 0))
			{
			g_ptr_array_add (array, comp);
				
			sqlite3_finalize (pstmt_icalstring);
			sqlite3_free (sql_str_icalstring);
			continue;	
			}
			
		str_exdate = (const gchar *)sqlite3_column_text(pstmt_icalstring,1); 
#ifdef DEBUG
	g_print ("%s():%d str_exdate = %s\n", __FUNCTION__, __LINE__, str_exdate);
#endif	
		 if (str_exdate != NULL)
		{
			array_exdate = _parse_exdate_string(str_exdate);
		#ifdef DEBUG
			int k;
			for (k=0; k<array_exdate->len; k++)
				g_print ("%s():%d array_exdate = %d\n", __FUNCTION__, __LINE__, *(time_t *)g_ptr_array_index(array_exdate, k));
		#endif
		}
	
		

		sqlite3_finalize (pstmt_icalstring);
		sqlite3_free (sql_str_icalstring);
		
		
		
		 
		/**for the master component, expand it according to start, end, rrule,exdate
		to get the instance rids **/
		/*If a component repeat forever, it's impossible to expand it if end == 0 */
		rid_generated = _icalrecur_expand_recurrence_by_time (comp, start, end);
		 /*remove the exdate_rids from rid_generated*/
		 if (str_exdate != NULL)
		{
			int i, j;
		 	for (i=0; i<array_exdate->len; i++)
			 	for(j=0; j<rid_generated->len; j++)
			 	{
				#ifdef DEBUG
					 g_print ("%s():%d array_exdate = %d\n", __FUNCTION__, __LINE__, *(time_t *)g_ptr_array_index(array_exdate, i));
				 	g_print ("%s():%d rid_generated = %d\n", __FUNCTION__, __LINE__, *(time_t *)g_ptr_array_index(rid_generated, j));
				#endif
					 if (*(time_t *)g_ptr_array_index(array_exdate, i) == *(time_t *)g_ptr_array_index(rid_generated, j))
				 	{
						g_free (g_ptr_array_index (rid_generated, j));
						g_ptr_array_remove_index (rid_generated, j);
				 	}
			 	}
		 }
		 if (rid_generated->len > 0) 
		 {
				 if (detached)
				 {
			 				/** construct instances from master component **/
							_fill_generated_occurrences (comp, rid_generated, array);
				 }
				 else
				 {
						 /*if we get the generated instance from the component by the time ,we not do following code and add the comp*/ 
						 g_ptr_array_add (array, comp);
						 continue;
				 }	 
		 }
		 
#ifdef DEBUG
	g_print ("%s():%d**********%d\n", __FUNCTION__, __LINE__, rid_generated->len);
#endif

		/*deal with the detached components*/
		gchar *temp_str = NULL;		
		temp_str = sqlite3_mprintf ("select icalstring from component where uid='%q' and rid NOTNULL ", str_uid);
		sql_str_rid = g_string_new (temp_str);
		g_free (temp_str);
		g_string_append (sql_str_rid, condition_str);
		sqlite3_prepare (db, sql_str_rid->str, -1, &pstmt_rid, 0);
		//int detached_count = 0;
		while (sqlite3_step (pstmt_rid) == SQLITE_ROW)
		{
				if (detached)
				{
						_fill_components(pstmt_rid, array);
				}
				else
				{
						/*if we find the detached instance from the component by the time ,we jump out from the while and add the comp*/ 
						g_ptr_array_add (array, comp);
						break;
				}
		}
		sqlite3_finalize (pstmt_rid);
		g_string_free (sql_str_rid, TRUE);
		
		if (detached)
			cal_component_free (comp);	
	}
	
	g_free (condition_str);
	sqlite3_finalize (pstmt_uid);
	g_free (sql_str_uid);
	return CAL_ERR_NONE;
}


cal_err_t cal_get_components_by_time_db (CalBackend *bk,
                                        time_t start,
                                        time_t end,
																				gboolean detached,
                                        cal_component_t ***components)
{
	
	CalBackendDb *priv = (CalBackendDb *)bk->priv;
	gint i;
	GPtrArray *array = g_ptr_array_new ();
	cal_component_t **tempcomps = NULL;
	cal_err_t err;
	
	if ((start == 0) && (end == 0) && (detached == TRUE))
		return CAL_ERR_INVALID;
	
  /*select non-repeating components*/
	err = _select_non_repeating_components (priv->component_db, start, end, array);	
	if (err != CAL_ERR_NONE)
		return err;
	/*select repeating components*/
	err = _select_repeating_components (priv->component_db, start, end, detached, array);	
	if (err != CAL_ERR_NONE)
		return err;
		
	
	if (array->len !=0)
	{
#ifdef DEBUG
	g_print ("%s():%d:%d\n", __FUNCTION__, __LINE__,array->len);
#endif		
		tempcomps = (cal_component_t **)g_new0 (gpointer, array->len+1);

		if (tempcomps == NULL)
		{
			g_print ("%s(): %d Memory allocation error\n", __FUNCTION__, __LINE__);
			return CAL_ERR_MEM_ALLOC;
		}

		for (i = 0; i<array->len; i++)			
			tempcomps[i] = g_ptr_array_index(array, i);		
		
		tempcomps[i] = NULL;
	}
	
	g_ptr_array_free (array, TRUE);	
	*components = tempcomps;
  return CAL_ERR_NONE;	
}



/*  For recurrence component, if an instance has been modified differrently from
*  other instances in the component, then this function will not apply to the 
*  differrent instance.
*/
cal_err_t cal_update_component_db (CalBackend *bk, cal_component_t *component)
{
	CalBackendDb *priv;
	const gchar *str_uid = NULL;
	const gchar *str_rid = NULL;
	cal_comp_type_t ctype;
	cal_err_t err;
	
	g_return_val_if_fail (component!= NULL, CAL_ERR_INVALID);
	
	cal_component_get_type (component, &ctype);

  switch (bk->type)
  {
    case CAL_TYPE_EVENT:
      if ((ctype != CAL_EVENT_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
        return CAL_ERR_INVALID;
      break;
    case CAL_TYPE_TODO:
      if ((ctype != CAL_TODO_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
          return CAL_ERR_INVALID;
      break;
    case CAL_TYPE_ALARM:
      if ((ctype != CAL_EVENT_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
          return CAL_ERR_INVALID;
      break;
    case CAL_TYPE_JOURNAL:
      if ((ctype != CAL_JOURNAL_COMPONENT) &&(ctype != CAL_TIMEZONE_COMPONENT))
          return CAL_ERR_INVALID;
      break;
  }

	priv = bk->priv;

/*UID should have been set when the component is inserted into database at the first time*/	
	
	cal_proptype_get_string_value (component, CAL_UID_PROPERTY, &str_uid);
	if (str_uid == NULL)
		return CAL_ERR_INVALID;
	
/*The instance should be updated by cal_update_instance()*/	
	cal_proptype_get_string_value (component, CAL_RECURRENCEID_PROPERTY, &str_rid);
	if (str_rid != NULL)
		return CAL_ERR_INVALID;
/*Don't remove detached component*/	
	err = _cal_remove_component_db (bk, str_uid, ONLY_MASTER_COMPONENT);

	if (err == CAL_ERR_NOT_FOUND)
			return err;
	
/*Don't call cal_add_component here, because the uid has been set*/
		err = _write_component_into_db (priv->component_db, component);

		if (err != CAL_ERR_NONE)
			return err;
	
	return CAL_ERR_NONE;
}


cal_err_t cal_update_instance_db (CalBackend *bk, const cal_component_t *instance, 
																	cal_mod_type_t modtype)
{
	const gchar *str_uid = NULL;
	const gchar *str_rid = NULL;
	CalBackendDb *priv = NULL;
	cal_err_t err;
	
	g_return_val_if_fail (instance != NULL, CAL_ERR_INVALID);
	priv = (CalBackendDb *)bk->priv;

	cal_proptype_get_string_value (instance, CAL_RECURRENCEID_PROPERTY, &str_rid);
	if (str_rid == NULL)
	{
		/*The instance may belongs to a non-recurrence component*/
#ifdef DEBUG		
		g_print ("%s():%d: The instance may belongs to a non-recurrence component!\n",
		__FUNCTION__, __LINE__);
#endif
		
		return cal_update_component_db (bk, instance);
	}
	
	cal_proptype_get_string_value (instance, CAL_UID_PROPERTY, &str_uid);
	
	if (str_uid == NULL)
	{
		g_print ("%s():%d: The uid isn't set in the instance\n", __FUNCTION__, __LINE__);	
		return CAL_ERR_INVALID;
	}
			
	if (modtype == CAL_MOD_THIS)
	{
		/*If there's an existing detached component for the instance , remove it from database*/
		err = cal_remove_instance_db (bk, str_uid, str_rid, CAL_MOD_THIS);
		if (err != CAL_ERR_NONE)
			return err;
		/*add detached component for the instance to database*/
		err = _write_component_into_db (priv->component_db, instance);
		if (err != CAL_ERR_NONE)
			return err;
	}
	else if (modtype == CAL_MOD_ALL)
	{
		cal_component_t *tempcomp;
		const char *rrule, *start, *end;
		
		cal_get_component_by_id_db (bk,  str_uid, NULL, &tempcomp);
		cal_proptype_get_string_value (tempcomp, CAL_RRULE_PROPERTY, &rrule);
		cal_proptype_get_string_value (tempcomp, CAL_DTSTART_PROPERTY, &start);
		cal_proptype_get_string_value (tempcomp, CAL_DTEND_PROPERTY, &end);
		
		err = cal_component_remove_property_by_type (instance, CAL_RECURRENCEID_PROPERTY);
		if (err != CAL_ERR_NONE)
			return CAL_ERR_INVALID;
	
		cal_proptype_set_string_value (instance, CAL_RRULE_PROPERTY, rrule);
		cal_proptype_set_string_value (instance, CAL_DTSTART_PROPERTY, start);
		cal_proptype_set_string_value (instance, CAL_DTEND_PROPERTY, end);
		cal_component_free (tempcomp);
		cal_remove_component_db (bk, str_uid);
		err = _write_component_into_db (priv->component_db, instance);
		if (err != CAL_ERR_NONE)
			return err;
	}
	else
	{
		/*FIXME: other modify type should be supported*/
		/*If modtype == CAL_MOD_THISANDFUTURE, following modifications should be taken:
		 1. The RRULE in master component should be set a UNTIL field
     2. A detached component with the RID should be insert into database
		*/
		g_print ("%s():%d: Only CAL_MOD_THIS and CAL_MOD_ALL is supported now!", __FUNCTION__, __LINE__);
	}
	
	return CAL_ERR_NONE;
}




cal_err_t _cal_remove_component_db (CalBackend *bk, const char *uid, rm_flag_t flag)
{	
	CalBackendDb *priv = NULL;
	gchar *sql_string = NULL;
	sqlite3_stmt *pstmt = NULL;	
	gint ret;
	
	g_return_val_if_fail (uid!=NULL , CAL_ERR_INVALID);
	priv = (CalBackendDb *)bk->priv;	

	/*check whether component with uid exists in backend*/
	sql_string = sqlite3_mprintf ("select * from component where uid='%q';", uid);
		
	ret = sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);	
	sqlite3_finalize (pstmt);
	sqlite3_free (sql_string);
	if (ret != SQLITE_ROW)
	{
		g_print ("Component with this uid does not exist\n");
		return CAL_ERR_NOT_FOUND;	
	}

	/*remove component from database*/
	if (flag == ALL_COMPONENT)
		sql_string = sqlite3_mprintf (REMOVE_COMPONENT_SQL, uid);	
	else if (flag == ONLY_MASTER_COMPONENT)
		sql_string = sqlite3_mprintf (REMOVE_MASTER_COMPONENT_SQL, uid);	
	ret = sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);
	sqlite3_finalize (pstmt);
	sqlite3_free (sql_string);	
	if (ret != SQLITE_DONE)
	{
		g_print ("%s():%d: Remove component error! ret=%d\n", __FUNCTION__, __LINE__, ret);
		return CAL_ERR_DB_TRANS;
	}

	return CAL_ERR_NONE;	
}


cal_err_t cal_remove_component_db (CalBackend *bk, const char *uid)
{
	return 	_cal_remove_component_db (bk, uid, ALL_COMPONENT);
}


/*FIXME: only deal with the situation where modtype = CAL_MOD_THIS or modtype = CAL_MOD_ALL*/
cal_err_t cal_remove_instance_db (CalBackend *bk, const char *uid, const char *rid,
																	cal_mod_type_t modtype)
{
	gchar *sql_string = NULL;
	sqlite3_stmt *pstmt = NULL;	
	cal_component_t *tempcomp = NULL;
	uint32_t exdate_pids;	
	CalBackendDb *priv = NULL;
	gint ret;
	
	g_return_val_if_fail (uid!=NULL, CAL_ERR_INVALID);
	priv = (CalBackendDb *)bk->priv;

	if (rid == NULL)
	{
		/*The instance may belongs to a non-recurrence component*/
#ifdef DEBUG		
		g_print ("%s():%d: The instance may belongs to a non-recurrence component!\n",
		__FUNCTION__, __LINE__);
#endif
		
		return cal_remove_component_db (bk, uid);
	}
	
	if ((modtype != CAL_MOD_THIS) &&(modtype != CAL_MOD_ALL))
	{
		g_print ("%s():%d: Only CAL_MOD_THIS and CAL_MOD_ALL are supported now\n", __FUNCTION__,__LINE__);	
		return CAL_ERR_INVALID;
	}

	if (modtype == CAL_MOD_THIS)
	{
	/*If there's detached component for this instance, remove the instance from database*/	
	sql_string = sqlite3_mprintf ("select * from component where uid='%q' AND rid='%q';", uid, rid);
	sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);
	sqlite3_finalize (pstmt);
	sqlite3_free (sql_string);
	
	if (ret == SQLITE_ROW)
	{	
		/*There's detached component. */
		sql_string = sqlite3_mprintf (REMOVE_INSTANCE_SQL, uid, rid);	
		sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
		ret = sqlite3_step (pstmt);	
		sqlite3_finalize (pstmt);
		sqlite3_free (sql_string);	
		if (ret != SQLITE_DONE)
			{
				g_print ("%s() :%d Remove instance error!\n", __FUNCTION__, __LINE__);
				return CAL_ERR_DB_TRANS;
			}
		return CAL_ERR_NONE;
	}
	else if (ret == SQLITE_DONE)
	{
	/*There's no detached component corresponding to the "uid" and "rid". Try to
		generate instance	from repeating component*/
		sql_string = sqlite3_mprintf ("select icalstring from component where uid='%q' AND " \
		                                 "(rid ISNULL);", uid);
		sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
		/*If the repeating component can't be found , there must be sth wrong*/
		ret = sqlite3_step (pstmt);

		if (ret != SQLITE_ROW)
			{
				sqlite3_finalize (pstmt);
				sqlite3_free (sql_string);
				return CAL_ERR_NOT_FOUND;
			}
		else
			{
			gchar *icalstring = NULL;
						
			icalstring = g_strdup ((gchar *)sqlite3_column_text (pstmt, 0));
			sqlite3_finalize (pstmt);
			sqlite3_free (sql_string);
				
			_cal_component_new_from_icalcomponent (&tempcomp, icalcomponent_new_from_string (icalstring));
			cal_component_add_property (tempcomp, CAL_EXDATE_PROPERTY, &exdate_pids);
			/*FIXME: We don't check the validity of "rid"here, the user should insure the validity of "rid"*/
			//cal_component_set_string_property_value (tempcomp, exdate_pids, rid);	
			cal_property_set_string_value (tempcomp, exdate_pids, rid);	
			/*Remove the old master recurrence component*/
			sql_string = sqlite3_mprintf ("delete from component where uid='%q' AND " \
		                                 "(rrule NOTNULL) AND (rid ISNULL);", uid);	
			sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
			sqlite3_step (pstmt);	
			sqlite3_finalize (pstmt);
			sqlite3_free (sql_string);	
			/*write the new master recurrence component into the database*/
			_write_component_into_db (priv->component_db, tempcomp);

			cal_component_free (tempcomp);				
			g_free (icalstring);
		}
	}
	else
		return CAL_ERR_DB_TRANS;
	return CAL_ERR_NONE;	
	}
	else if (modtype == CAL_MOD_ALL)
	{
		return cal_remove_component_db (bk, uid);
	}
}


cal_err_t cal_instance_is_multiple_db (CalBackend *bk, const cal_component_t *instance,
                                    cal_boolean_t *bool)
{
	CalBackendDb *priv = NULL;
	const gchar *uid = NULL;	
	gchar *sql_string = NULL;
	sqlite3_stmt *pstmt = NULL;	
	const gchar *value = NULL;
	gint ret;
	cal_err_t err;
	
	g_return_val_if_fail (instance != NULL, CAL_ERR_INVALID);
	priv = (CalBackendDb *)bk->priv;
	*bool = FALSE;
	
	cal_proptype_get_string_value (instance, CAL_UID_PROPERTY, &uid);
	if (uid == NULL)
	{
		g_print ("%s():%d: The input wrong!\n",__FUNCTION__, __LINE__);
		return CAL_ERR_INVALID;
	}
	
	sql_string = sqlite3_mprintf ("select rrule from component where uid='%q';", uid);
	sqlite3_prepare (priv->component_db, sql_string, -1, &pstmt, 0);
	while ((ret = sqlite3_step (pstmt)) == SQLITE_ROW)
		{
		value = (gchar *)sqlite3_column_text (pstmt, 0);
		if (value)
			{		
			*bool = TRUE;
			break;
			}	
		}
	sqlite3_finalize (pstmt);
	sqlite3_free (sql_string);
	

	return CAL_ERR_NONE;
}
/*Called by the function "cal_get_component_by_id_db()".
  In this function, get component only by uid*/
cal_err_t _cal_get_component_by_uid_db (CalBackend *bk, const gchar *uid, 
																				cal_component_t **component)
{
	CalBackendDb *priv;
	gchar *sql_str = NULL;
	sqlite3_stmt *pstmt;
	cal_component_t *comp = NULL;	
	gchar *value = NULL;
	gint ret; 
	
	g_return_val_if_fail (uid!=NULL, CAL_ERR_INVALID);
	priv = bk->priv;
	sql_str = sqlite3_mprintf ("select icalstring from component where uid='%q'", uid);
	sqlite3_prepare (priv->component_db, sql_str, -1, &pstmt, 0);
	ret = sqlite3_step (pstmt);
	if (ret == SQLITE_DONE)
		{
			sqlite3_finalize (pstmt);
			sqlite3_free (sql_str);
			return CAL_ERR_NOT_FOUND;
		}
	else if (ret == SQLITE_ROW)
		{
			value = (gchar *)sqlite3_column_text (pstmt, 0);
			_cal_component_new_from_icalcomponent (&comp, icalcomponent_new_from_string (value));
			*component = comp;
			
			sqlite3_finalize (pstmt);
			sqlite3_free (sql_str);
		}	
	
		return CAL_ERR_NONE;
}



/*Generate an instance of "rid" from component*/
cal_component_t * 
_get_instance_from_component(cal_component_t *comp, const char *rid)
{
	struct icalrecurrencetype recur; 					// Recurrence struct
	icalrecur_iterator* ritr;									// An iterator on the recurrence
  time_t tt;			// Used as the time of the current instance of the recurrence
	time_t rid_time; //Time of the rid
  struct icaltimetype compstart, next;			
  /*instance should be assigned NULL when declared*/
  cal_component_t *instance = NULL;		
	

	rid_time = icaltime_as_timet (icaltime_from_string (rid));
#ifdef DEBUG
	g_print ("%s():%d  rid=%s,  rid_time=%d\n", __FUNCTION__, __LINE__, rid, rid_time);
#endif
	
	
	compstart = icalproperty_get_dtstart (icalcomponent_get_first_property(comp->icalcomp, ICAL_DTSTART_PROPERTY));
	
	const icalproperty* prop= NULL;
	
	prop = icalcomponent_get_first_property(comp->icalcomp, ICAL_RRULE_PROPERTY);
#ifdef DEBUG
	g_print ("%s():%d, comp->icalcomp = %s\n", __FUNCTION__, __LINE__,icalcomponent_as_ical_string(comp->icalcomp) );
#endif	
	if (prop)
		recur = icalproperty_get_rrule (prop);
	
	ritr = icalrecur_iterator_new(recur,compstart);	// Initialize the iterator with the recur and the start time of the component
    
	next = icalrecur_iterator_next(ritr);	// Set the iterator to next valid time according to recurrence
	tt = icaltime_as_timet(next);

	//while(	!icaltime_is_null_time(next))
  while (tt != 0)
		{
		/*The instance start time == rid_time*/
		if (tt == rid_time)
			{	
			_fill_generated_occurrence(comp, rid, &instance);
				
#ifdef DEBUG 
	g_print ("%s():%d: find the instance\n", __FUNCTION__, __LINE__);
#endif				
				break;
			}
		next = icalrecur_iterator_next(ritr);	// Go to next instance 
		tt = icaltime_as_timet(next);
		}

		
	icalrecur_iterator_free(ritr);

	return instance;
}

/*Called by the function "cal_get_component_by_id_db()".
  In this function, get instance by uid and rid*/
cal_err_t _cal_get_instance_by_rid_db (CalBackend *bk, const gchar *uid, 
																			 const char *rid, cal_component_t **component)
{
	CalBackendDb *priv;
	/*To differentiate with later sql_str and pstmt,give them prefix like "first, second ..."*/
	gchar *first_sql_str = NULL;																
	sqlite3_stmt *first_pstmt;
	gint ret; 
	
	g_return_val_if_fail (uid!=NULL, CAL_ERR_INVALID);
	g_return_val_if_fail (rid!=NULL, CAL_ERR_INVALID);
	
	priv = bk->priv;
	
	first_sql_str = sqlite3_mprintf ("select icalstring from component where uid='%q' AND rid='%q';", uid, rid);
	sqlite3_prepare (priv->component_db, first_sql_str, -1, &first_pstmt, 0);
	ret = sqlite3_step (first_pstmt);
	if (ret == SQLITE_ROW)
		{
		const gchar *value = NULL;
			
		/*There's a detached component in database for the uid and rid.*/
		value = (gchar *)sqlite3_column_text (first_pstmt, 0);
		_cal_component_new_from_icalcomponent (component, icalcomponent_new_from_string (value));
				
		sqlite3_finalize (first_pstmt);
		g_free (first_sql_str);
	
		return CAL_ERR_NONE;
		}
	else if (ret == SQLITE_DONE)
		{
		gchar *sec_sql_str = NULL;
		sqlite3_stmt *sec_sql_pstmt;
			
		/*There's no detached component. Try to generate instance from the master component.*/	
		sqlite3_finalize (first_pstmt);
		g_free (first_sql_str);
	
		sec_sql_str = sqlite3_mprintf ("select icalstring from component where uid='%q' and (rrule NOTNULL);", uid);
#ifdef DEBUG
		g_print ("%s():%d, select_str=%s\n", __FUNCTION__, __LINE__, sec_sql_str);
#endif			
		sqlite3_prepare (priv->component_db, sec_sql_str, -1, &sec_sql_pstmt, 0);
		ret = sqlite3_step (sec_sql_pstmt);
		if (ret == SQLITE_DONE)
			{
			sqlite3_finalize (sec_sql_pstmt);
			g_free (sec_sql_str);
			return CAL_ERR_NOT_FOUND;
			}
		else if (ret == SQLITE_ROW)
			{
			const gchar *value = NULL;
			cal_component_t *tempcomp;
			cal_err_t err;
				
			value = (const gchar *)sqlite3_column_text (sec_sql_pstmt, 0);
			/*calculate whether the repeating component has an instance whose recurrence id = rid*/
			_cal_component_new_from_icalcomponent (&tempcomp, icalcomponent_new_from_string (value));
			*component = _get_instance_from_component (tempcomp, rid);		
			
			cal_component_free (tempcomp);
			sqlite3_finalize (sec_sql_pstmt);
			g_free (sec_sql_str);
				
			if (*component)
				return CAL_ERR_NONE;
			else
				return CAL_ERR_NOT_FOUND;
		
			}
		}			
}

cal_err_t cal_get_component_by_id_db (CalBackend *bk, const gchar *uid, 
																			const char *rid, cal_component_t **component)
{
		
	if (rid == NULL)
	{
		/*query component only by uid*/
		return _cal_get_component_by_uid_db (bk, uid, component);
	}
	else
	{
		/*if both uid and rid are given*/
		return _cal_get_instance_by_rid_db (bk, uid, rid, component);
	}
		
}




cal_err_t cal_funcs_init_db (CalBackend *bk)
{
	g_return_val_if_fail (bk!=NULL, CAL_ERR_INVALID);	
  
  bk->funcs = g_new0 (CalendarFuncs, 1);
  if (bk->funcs == NULL)
  {
    return CAL_ERR_MEM_ALLOC;
  }
	

  bk->funcs->cal_initialize = cal_initialize_db;
  bk->funcs->cal_finalize = cal_finalize_db;
	
  bk->funcs->cal_add_component = cal_add_component_db;

	bk->funcs->cal_get_components_by_time = cal_get_components_by_time_db;
	bk->funcs->cal_remove_component = cal_remove_component_db;		
	bk->funcs->cal_update_component = cal_update_component_db;	
	bk->funcs->cal_get_component_by_id = cal_get_component_by_id_db; 
	bk->funcs->cal_update_instance = cal_update_instance_db;	
	bk->funcs->cal_remove_instance = cal_remove_instance_db;
	bk->funcs->cal_instance_is_multiple=cal_instance_is_multiple_db;
  return CAL_ERR_NONE;
	
}
